/*******************************************************************************
 * Copyright 2008 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the Software), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/



using System;
using System.Collections.Generic;
using System.Text;

namespace TTArch
{
    class Engine
    {
        //TestTrack project variables
        public string ttSrc;
        public string ttDest;
        public string ttProject;


        //static vars
        public string ttUser = "soap";
        public string ttPassword = "passoap";
        public string ttURL = "http://localhost/ttsoapcgi.exe";
        
        //SOAP vars
        public ttsoapcgi m_Engine;
        public long m_lSession;

        public string ErrMsg;

        public Engine()
        {
            //Define our soap object
            m_Engine = new ttsoapcgi();

        }
        public string MoveSingleDefect(long m_defNum)
        {
            ErrMsg = "";
            //Define defect object that we are going to move
            CDefect defectToMove = new CDefect();
            //set the current project variable to the source project
            SetCurProject(ttSrc);
            if (Login())
            {
                try
                {
                    //get defect object to add to new project
                    defectToMove = m_Engine.getDefect(m_lSession, m_defNum, "", false);
                    //once we have the defect object in memory, delete it from project
                    m_Engine.deleteDefect(m_lSession, m_defNum, "");
                }
                catch (Exception f)
                {
                    ErrMsg = f.Message;
                }
                if (m_lSession > 0)
                {
                    try
                    {
                        //logout from the source project
                        m_Engine.DatabaseLogoff(m_lSession);
                        //reset the Cookie
                        m_lSession = 0;
                    }
                    catch (Exception f)
                    {
                        ErrMsg = ErrMsg + "\n" + f.Message;
                    }
                }
                if (ErrMsg == "")
                {
                    //Set the current project to the destination project
                    SetCurProject(ttDest);
                    if (Login())
                    {
                        try
                        {
                            //Add the defect
                            m_Engine.addDefect(m_lSession, defectToMove);
                            Console.WriteLine("Moving Defect {0}", m_defNum);
                        }
                        catch (Exception f)
                        {
                            ErrMsg = f.Message;
                        }
                        if (m_lSession > 0)
                        {
                            try
                            {
                                //Logout from the destination project
                                m_Engine.DatabaseLogoff(m_lSession);
                                //Reset the Cookie
                                m_lSession = 0;
                            }
                            catch (Exception f)
                            {
                                ErrMsg = ErrMsg + "\n" + f.Message;
                            }
                        }
                    }
                    else
                    {
                        ErrMsg = "Unable to log in to the destination project";
                    }
                   
                }
                else
                {
                    return ErrMsg;
                }
            }
            else
            {
                ErrMsg = "Unable to log in to source project!";
            }
            return ErrMsg;

        }
        public string MoveDefects(string m_ttFilter)
        {
            ErrMsg = "";
            //Define object that will get list of defects
            CRecordListSoap m_ttReclist = null;
            //set current project to Source project
            SetCurProject(ttSrc);
            if (Login())
            {
                //build a column array that only contains the one element we care about: Defect Number
                CTableColumn[] m_ttColumns = new CTableColumn[1];
                m_ttColumns[0] = new CTableColumn();
                m_ttColumns[0].name = "Number";
                //Get the list of tables
                CDatabaseTable[] m_ttTables = m_Engine.getTableList(m_lSession);
                try
                {
                    //Get list of defects numbers returned by filter and columb array
                    m_ttReclist = m_Engine.getRecordListForTable(m_lSession, m_ttTables[0].name, m_ttFilter, m_ttColumns);
                }
                catch (Exception f)
                {
                    ErrMsg = f.Message;
                    return ErrMsg;
                }
            }
            else
            {
                ErrMsg = "There was a problem logging in";
                return ErrMsg;
            }
            if (m_lSession > 0)
            {
                m_Engine.DatabaseLogoff(m_lSession);
                m_lSession = 0;
            }
            //If we have defect data, then's let's go on to moving defects!!!
            if ((!(m_ttReclist == null)) && (m_ttReclist.records.Length > 0))
            {
                //for each defect number, move the defect
                for (int i = 0; i < m_ttReclist.records.Length; i++)
                {
                    ErrMsg = MoveSingleDefect(Convert.ToInt64(m_ttReclist.records[i].row[0].value));

                }
            }
            else
            {
                ErrMsg = "The filter did not return any defects!";
            }
            return ErrMsg;
        }
        private bool Login()
        {

            CProject m_ttProject = new CProject();

            CDatabase ProjDB = new CDatabase();
            ProjDB.name = ttProject;

            CProjectDataOption[] ProjOps = new CProjectDataOption[1];
            CProjectDataOption TestTrackOp = new CProjectDataOption();
            TestTrackOp.name = "TestTrack Pro";
            ProjOps.SetValue(TestTrackOp, 0);

            m_ttProject.database = ProjDB;
            m_ttProject.options = ProjOps;
            try
            {
                m_lSession = m_Engine.ProjectLogon(m_ttProject, ttUser, ttPassword);
            }
            catch
            {
                return (false);
            }
            if (m_lSession > 0)
            {
                return (true);
            }
            else
            {
                return (false);
            }
        }
        private void SetCurProject(string ttProjName)
        {
            ttProject = ttProjName;
        }

    }
}
